# -*- coding: utf-8 -*-

"""Provide ``request.redirect_to(location)``."""

import logging
logger = logging.getLogger(__name__)

from pyramid.httpexceptions import HTTPFound
from pyramid.httpexceptions import HTTPMovedPermanently

def get_redirect_to(request, location, permanently=False, **kwargs):
    """Return a redirect response to the location provided."""
    
    # Compose.
    redirect_cls = HTTPMovedPermanently if permanently else HTTPFound
    return redirect_cls(location=location, **kwargs)




# -*- coding: utf-8 -*-

"""Provides a ``QueueProcessor`` utility that consumes and processes data
  from one or more redis channels.
  
      >>> redis_client = '<redis.Redis> instance'
      >>> input_channels = ['channel1', 'channeln']
      >>> handle_data = lambda data_str: data_str
      >>> processor = QueueProcessor(redis_client, input_channels, handle_data)
  
  Run in the main / current thread::
  
      >>> # processor.start()
  
  Run in a background thread::
      
      >>> # processor.start(async=True)
      
  If running in a background thread, call ``stop()`` to exit::
  
      >>> # processor.stop()
  
  If you want jobs to be requeued (at the back of the queue)
  This provides a very simple inter-process messaging and / or background
  task processing mechanism.  Queued messages / jobs are explictly passed
  as string messages.  
  
  Pro: you're always in control of your code execution environment.
  Con: you have to deal with potentially tedious message parsing.
"""

import logging
logger = logging.getLogger(__name__)

import json
import threading
import time

class QueueProcessor(object):
    """Consume data from a redis queue.  When it arrives, pass it to
      ``self.handler_function``.
    """
    
    running = False
    
    def stop(self):
        """Call ``stop()`` to stop processing the queue the next time a job is
          processed or the input queue timeout is reached.
        """
        
        logger.info('QueueProcessor.stop()')
        
        self.running = False
    
    def _start(self):
        """Actually start processing the input queue(s) ad-infinitum."""
        
        logger.info('QueueProcessor.start()')
        logger.info(self.channels)
        
        self.running = True
        while self.running:
            logger.debug(('QueueProcessor reconnecting', self.channels))
            try:
                return_value = self.redis.blpop(self.channels, timeout=self.timeout)
            except Exception as err:
                logger.warn(err, exc_info=True)
                time.sleep(self.timeout)
            else:
                logger.debug(('QueueProcessor return value obtained', self.channels))
                if return_value is not None:
                    channel, body = return_value
                    try:
                        self.handle_function(body)
                    except Exception as err:
                        logger.warn(err, exc_info=True)
                        logger.warn(return_value)
                        if self.should_requeue:
                            self.redis.rpush(channel, body)
                if self.reconnect_delay:
                    time.sleep(self.reconnect_delay)
    
    def start(self, async=False):
        """Either start running or start running in a thread."""
        
        if self.running:
            return
        
        if async:
            threading.Thread(target=self._start).start()
        else:
            self._start()
    
    def __init__(self, redis_client, channels, handle_function, timeout=20,
            reconnect_delay=0.005, should_requeue=False):
        """Instantiate a queue processor::
          
              >>> processor = QueueProcessor(None, None, None)
          
        """
        
        self.redis = redis_client
        self.channels = channels
        self.handle_function = handle_function
        self.timeout = timeout
        self.reconnect_delay = reconnect_delay
        self.should_requeue = should_requeue
    




# -*- coding: utf-8 -*-

"""Library patches."""

import logging
logger = logging.getLogger(__name__)

import hashlib
import sys

from beaker import cache
from beaker.util import coerce_cache_params
try: # pyramid >= 1.5
    import pyramid_mako
except ImportError: # <= 1.4
    from pyramid import mako_templating as pyramid_mako

from mako.cache import register_plugin
from mako.lookup import TemplateLookup

try:
    from dogpile.cache import make_region
    from dogpile.cache.plugins.mako_cache import MakoPlugin
except ImportError:
    pass
else:
    class UnicodeKeyCapableMakoPlugin(MakoPlugin):
        """Override the plugin to handle unicode keys gracefully."""
        
        def _prepare(self, key):
            """Coerce to UTF-8 and hash if longer than 250 chars."""
            
            k = key.encode('utf-8') if isinstance(key, unicode) else key
            return hashlib.md5(k).hexdigest() if len(k) > 250 else k
        
        def get_and_replace(self, key, *args, **kw):
            key = self._prepare(key)
            return MakoPlugin.get_and_replace(self, key, *args, **kw)

        def get_or_create(self, key, *args, **kw):
            key = self._prepare(key)
            return MakoPlugin.get_or_create(self, key, *args, **kw)

        def put(self, key, *args, **kw):
            key = self._prepare(key)
            return MakoPlugin.put(self, key, *args, **kw)

        def get(self, key, *args, **kw):
            key = self._prepare(key)
            return MakoPlugin.get(self, key, *args, **kw)

        def invalidate(self, key, *args, **kw):
            key = self._prepare(key)
            return MakoPlugin.invalidate(self, key, *args, **kw)
        
    

def coerge_dogpile_args(args):
    """Coerce INI settings into cache args with a ``default`` dogpile region.
      
      Requires args having at least:
      - url
      - type
      - expire
      
      XXX replace this as and when you actually want multiple regions.
    """
    
    # If the url isn't a list, make it so.
    url = args['url']
    if isinstance(url, basestring):
        url = [url]
    
    # Build the backend args.
    backend = args['type']
    backend_kwargs = {
        'url': [args['url']],
    }
    expiration_time = args['expire']
    
    # With optional auth.
    username = args.get('username', None)
    if username is not None:
        backend_kwargs['username'] = username
    password = args.get('password', None)
    if password is not None:
        backend_kwargs['password'] = password
    
    # Make the mako cache happy dogpile region.
    region = make_region().configure(
        backend,
        expiration_time=expiration_time,
        arguments=args
    )
    return {'regions': {'default': region}}

    
def templateLookupFactory(settings):
    """Use the ``settings`` to return a patched ``TemplateLookup`` class."""
    
    class TemplateLookup(pyramid_mako.PkgResourceTemplateLookup):
        """Sets ``cache_args`` if not passed into the lookup constructor."""
    
        def __init__(self, *args, **kwargs):
            """Patch ``kwargs['cache_args']``."""
            
            if not 'cache_args' in kwargs:
                # Read the ``cache.*`` ones into ``cache_args``.
                cache_args = {}
                prefix = 'mako.cache_args.'
                for key in settings.keys():
                    if key.startswith(prefix):
                        name = key.split(prefix)[1].strip()
                        value = settings[key]
                        try:
                            value = value.strip()
                        except AttributeError:
                            pass
                        cache_args[name] = value
                coerce_cache_params(cache_args)
                cache_args['timeout'] = cache_args.get('expire')
                type_arg = cache_args.get('type')
                if type_arg and type_arg.startswith('dogpile.cache'):
                    cache_args = coerge_dogpile_args(cache_args)
                    register_plugin("dogpile.cache",
                            "pyramid_weblayer.patch", "UnicodeKeyCapableMakoPlugin")
                    kwargs['cache_impl'] = 'dogpile.cache'
                kwargs['cache_args'] = cache_args
            super(TemplateLookup, self).__init__(*args, **kwargs)
        
    
    return TemplateLookup


def patch_all(settings):
    """Run all the patches."""
    
    # Patch ``mako.lookup.TemplateLookup`` to be our ``TemplateLookup`` class.
    pyramid_mako.PkgResourceTemplateLookup = templateLookupFactory(settings)




#!/usr/bin/env python
# -*- coding: utf-8 -*-

from pyramid.events import BeforeRender, ContextFound, NewRequest, NewResponse
from pyramid.settings import asbool

from .campaign import get_campaign_url
from .csrf import validate_against_csrf
from .flash import get_joined_flash
from .flat import add_flatten_functions
from .hsts import hsts_redirect_to_https
from .hsts import set_hsts_header
from .hsts import secure_application_url
from .hsts import secure_resource_url
from .hsts import secure_route_url
from .i18n import add_underscore_translation
from .markdown import markdown_to_html
from .nav import add_is_active_function
from .redirect import get_redirect_to
from .seen import set_seen_cookie
from .seen import get_has_been_seen
from .serve import get_serve_spec
from .session import get_session_id
from .snip import add_snip_functions
from .track import get_track_event
from .track import get_track_page
from .utils import *

def includeme(config):
    """Allow developers to use ``config.include('pyramid_weblayer')`` to register
      the ``add_underscore_translation`` subscriber::
      
      Setup::
      
          >>> from mock import Mock
          >>> mock_config = Mock()
          >>> includeme(mock_config)
      
      CSRF validation::
      
          >>> mock_config.add_subscriber.assert_any_call(validate_against_csrf, 
          ...         ContextFound)
      
      Provide `_` template namespace::
      
          >>> mock_config.add_subscriber.assert_any_call(add_underscore_translation,
          ...         BeforeRender)
          >>> mock_config.add_subscriber.assert_any_call(add_is_active_function,
          ...         BeforeRender)
      
      Add flatten and snip functions to template namespace::
      
          >>> mock_config.add_subscriber.assert_any_call(add_flatten_functions,
          ...         BeforeRender)
          >>> mock_config.add_subscriber.assert_any_call(add_snip_functions,
          ...         BeforeRender)
      
      Optionally force https::
      
          >>> mock_config.include.called
          False
          >>> mock_config.registry.settings = {'hsts.force_https': True}
          >>> includeme(mock_config)
          >>> mock_config.include.assert_any_call('pyramid_hsts')
      
      Has been seen flag.::
      
          >>> mock_config.add_subscriber.assert_any_call(set_seen_cookie, 
          ...         NewResponse)
          >>> mock_config.set_request_property.assert_any_call(get_has_been_seen,
          ...         'has_been_seen', reify=True)
      
      Session ID::
      
          >>> mock_config.set_request_property.assert_any_call(get_session_id, 
          ...         'session_id', reify=True)
      
      Joined session flash::
      
          >>> mock_config.set_request_property.assert_any_call(get_joined_flash, 
          ...         'joined_flash', reify=True)
      
      Markdown rendering::
      
          >>> mock_config.set_request_property.assert_any_call(markdown_to_html, 
          ...         'markdown_to_html', reify=True)
      
      Track event::
      
          >>> mock_config.set_request_property.assert_any_call(get_track_event, 
          ...         'track_event', reify=True)
      
      
      Prereq routes::
      
          >>> mock_config.add_route.assert_any_call('favicon_ico', 'favicon.ico')
          >>> mock_config.add_route.assert_any_call('robots_txt', 'robots.txt')
      
      Scan::
      
          >>> mock_config.scan.assert_called_with('pyramid_weblayer')
      
    """
    
    # CSRF validation.
    config.add_subscriber(validate_against_csrf, ContextFound)
    
    # Provide `_` template namespace.
    config.add_subscriber(add_underscore_translation, BeforeRender)
    config.add_subscriber(add_is_active_function, BeforeRender)
    
    # Add snip and flatten functions to the template namespace.
    config.add_subscriber(add_flatten_functions, BeforeRender)
    config.add_subscriber(add_snip_functions, BeforeRender)
    
    # Optionally force https://
    settings = config.registry.settings
    should_force_https = asbool(settings.get('hsts.force_https', False))
    if should_force_https:
        config.include('pyramid_hsts')
    
    # Has been seen flag.
    config.add_subscriber(set_seen_cookie, NewResponse)
    config.set_request_property(get_has_been_seen, 'has_been_seen', reify=True)
    
    # Session id.
    config.set_request_property(get_session_id, 'session_id', reify=True)
    
    # Provide ``request.campaign_url``.
    config.set_request_property(get_campaign_url, 'campaign_url', reify=True)
    
    # Provide ``request.joined_flash``.
    config.set_request_property(get_joined_flash, 'joined_flash', reify=True)
    
    # Provide ``request.markdown_to_html``.
    config.set_request_property(markdown_to_html, 'markdown_to_html', reify=True)
    
    # Provide ``request.redirect_to``.
    config.add_request_method(get_redirect_to, 'redirect_to')
    
    # Provide ``request.serve_spec``.
    config.set_request_property(get_serve_spec, 'serve_spec', reify=True)
    
    # Provide ``request.track_event``.
    config.set_request_property(get_track_event, 'track_event', reify=True)
    config.set_request_property(get_track_page, 'track_page', reify=True)
    
    # Favicon and robots.txt.
    config.add_route('favicon_ico', 'favicon.ico')
    config.add_route('robots_txt', 'robots.txt')
    config.scan('pyramid_weblayer')




# -*- coding: utf-8 -*-

"""Provides a base `layout class`_ used to consolidate the api provided
  to templates.
  
  _`layout classes`: http://pyramid_layout.readthedocs.org
"""

__all__ = [
    'Layout'
]

import logging
logger = logging.getLogger(__name__)

from pyramid.security import has_permission

class Layout(object):
    """A simple base layout api implementation."""
    
    def has_permission(self, permission, context, has_perm=None):
        """Can the current request (i.e.: the authenticated user if any)
          ``permission`` the ``context``?
        """
        
        # Compose.
        if has_perm is None:
            has_perm = has_permission
        
        # Return the verdict.
        return has_perm(permission, context, self.request)
    
    def can_admin(self, context, has_perm=None):
        return self.has_permission('admin', context)
    
    def can_create(self, context, has_perm=None):
        return self.has_permission('create', context)
    
    def can_delete(self, context, has_perm=None):
        return self.has_permission('delete', context)
    
    def can_edit(self, context, has_perm=None):
        return self.has_permission('edit', context)
    
    def can_view(self, context, has_perm=None):
        return self.has_permission('view', context)
    
    def __init__(self, context, request):
        self.request = request
        self.context = request.context
        self.registry = request.registry
        self.settings = request.registry.settings
        self.home_url = request.application_url
    




# -*- coding: utf-8 -*-

"""Provide ``get_session_id`` function."""

import logging
logger = logging.getLogger(__name__)

from .utils import generate_random_digest

def get_session_id(request, key='session_id', gen_digest=None):
    """Make sure there's a ``session_id`` in ``request.session`` and return it.
      
      Setup::
      
          >>> from mock import Mock
          >>> mock_request = Mock()
          >>> mock_gen_digest = Mock()
          >>> mock_gen_digest.return_value = '<digest>'
      
      If there's no existing id in the session, then generate, store and return::
      
          >>> mock_request.session = {}
          >>> get_session_id(mock_request, key='k', gen_digest=mock_gen_digest)
          '<digest>'
          >>> mock_request.session['k'] = '<digest>'
      
      Otherwise return the xisting value::
      
          >>> mock_request.session = {'k': '<existing>'}
          >>> get_session_id(mock_request, key='k')
          '<existing>'
      
    """
    
    # Test jig.
    if gen_digest is None:
        gen_digest = generate_random_digest
    
    digest = request.session.get(key)
    if not digest:
        digest = gen_digest()
        request.session[key] = digest
    return digest




# -*- coding: utf-8 -*-

"""Provides a function to render markdown as html."""

import logging
logger = logging.getLogger(__name__)

import markdown2

def markdown_to_html(request, to_html=None):
    """Return a function that renders markdown as html."""
    
    # Compose.
    if to_html is None:
        to_html = markdown2.markdown
    
    # If ``None`` return ''
    def render_as_html(markdown_str):
        """Render ``markdown_str`` as html."""
        
        # Coerce ``None`` to ``u''``.
        if markdown_str is None:
            markdown_str = u''
        
        return to_html(markdown_str)
    
    return render_as_html




# -*- coding: utf-8 -*-

"""Provides a ``join_to_transaction`` function which adds a function call,
  with args / kwargs, as an after commit hook.
"""

import logging
logger = logging.getLogger(__name__)

import threading
import transaction

def _handle_commit(tx_succeeded, *args_list, **kwargs):
    """Handle a successful commit by calling the callable registered as an after
      commit hook.
      
      Setup::
      
          >>> from mock import Mock
          >>> mock_callable = Mock()
          >>> mock_args_list = [mock_callable, 'a', 'b']
      
      If the transaction didn't succeed, it's a noop::
      
          >>> _handle_commit(False)
          >>> mock_callable.called = False
      
      If ``tx_succeeded`` is callable, treats the ``args_list`` as the remaining args::
      
          >>> _handle_commit(mock_callable, *mock_args_list[1:], foo='bar')
          >>> mock_callable.assert_called_with('a', 'b', foo='bar')
      
      Otherwise treats the first item in the ``args_list`` as the callable
      and calls it with the remaining args and the kwargs::
      
          >>> _handle_commit(True, *mock_args_list, foo='bar')
          >>> mock_callable.assert_called_with('a', 'b', foo='bar')
      
    """
    
    # If in debug mode, show what's going on under the hood.
    log_args = (tx_succeeded, args_list, kwargs)
    logger.debug('handle_commit: {0} for {1} {2}'.format(*log_args))
    
    # If the transaction failed, return.
    if tx_succeeded is False:
        return
    
    # Otherwise, if the first arg to this function is a callable,
    # this function was trigged by a before commit hook, so unpack accordingly.
    if callable(tx_succeeded):
        callable_ = tx_succeeded
        args = args_list
    else: # this came from an after commit hook so again unpack accordingly.
        callable_ = args_list[0]
        args = args_list[1:]
    
    # Call the callable with the unpacked args and kwargs.
    callable_(*args, **kwargs)

def _join_to_transaction(when, callable_, *args, **kwargs):
    """Add an after commit hook to the current transaction to call 
      ``callable_(*args, **kwargs)`` iff the current transaction succeeds.
      
      Setup::
      
          >>> from mock import Mock
          >>> import transaction
          >>> _original_get = transaction.get
          >>> transaction.get = Mock()
          >>> mock_tx = Mock()
          >>> transaction.get.return_value = mock_tx
      
      Add an after commit hook to the current transaction::
      
          >>> _join_to_transaction('after', 'callable', 'a', 'b', foo='bar')
          >>> mock_tx.addAfterCommitHook.assert_called_with(_handle_commit, 
          ...         args=['callable', 'a', 'b'], kws={'foo': 'bar'})
      
      Add a before commit hook::
      
          >>> mock_tx = Mock()
          >>> transaction.get.return_value = mock_tx
          >>> _join_to_transaction('before', 'callable', 'a', 'b', foo='bar')
          >>> mock_tx.addBeforeCommitHook.assert_called_with(_handle_commit, 
          ...         args=['callable', 'a', 'b'], kws={'foo': 'bar'})
      
      Note that ``'after'`` and ``'before'`` are the only ``when`` values currently
      implemented::
      
          >>> _join_to_transaction('foo', 'callable', 'a', 'b', foo='bar')
          Traceback (most recent call last):
          ...
          NotImplementedError
      
      Teardown::
      
          >>> transaction.get = _original_get
      
    """
    
    # If in debug mode, show what's going on under the hood.
    log_args = (callable_, args, kwargs)
    logger.debug('join_to_transaction: {0} {1} {2}'.format(*log_args))
    
    # Build a args list of [callable_, *args]
    args_list = [callable_]
    args_list.extend(args)
    
    # Add an after or before commit hook.
    current_tx = transaction.get()
    if when == 'before':
        method_name = 'addBeforeCommitHook'
    elif when == 'after':
        method_name = 'addAfterCommitHook'
    else:
        raise NotImplementedError
    method = getattr(current_tx, method_name)
    method(_handle_commit, args=args_list, kws=kwargs)


def join_before_transaction(callable_, *args, **kwargs):
    """Call ``callable_(*args, **kwargs)`` before the current transaction commits.
      
      Setup::
      
          >>> from mock import Mock
          >>> from pyramid_weblayer import tx
          >>> _original = tx._join_to_transaction
          >>> tx._join_to_transaction = Mock()
          >>> tx._join_to_transaction.return_value = None
      
      Test::
      
          >>> join_before_transaction('func', 'arg', kw='')
          >>> tx._join_to_transaction.assert_called_with('before', 'func', 'arg', kw='')
      
      Teardown::
      
          >>> tx._join_to_transaction = _original
      
    """
    
    return _join_to_transaction('before', callable_, *args, **kwargs)

def join_after_transaction(callable_, *args, **kwargs):
    """Call ``callable_(*args, **kwargs)`` after the current transaction commits.
      
      Setup::
      
          >>> from mock import Mock
          >>> from pyramid_weblayer import tx
          >>> _original = tx._join_to_transaction
          >>> tx._join_to_transaction = Mock()
          >>> tx._join_to_transaction.return_value = None
      
      Test::
      
          >>> join_after_transaction('func', 'arg', kw='')
          >>> tx._join_to_transaction.assert_called_with('after', 'func', 'arg', kw='')
      
      Teardown::
      
          >>> tx._join_to_transaction = _original
      
    """
    
    return _join_to_transaction('after', callable_, *args, **kwargs)


# Default the main ``join_to_transaction`` to use an after commit hook.
join_to_transaction = join_after_transaction

def call_in_background(target, args=None, kwargs=None, join=None, thread_cls=None):
    """Helper function for the common use case of firing and forgetting a
      function call in a background thread.
    """
    
    # Compose.
    if join is None:
        join = join_to_transaction
    if thread_cls is None:
        thread_cls = threading.Thread
    
    thread_kwargs = {}
    if args is not None:
        thread_kwargs['args'] = args
    if kwargs is not None:
        thread_kwargs['kwargs'] = kwargs
    thread = thread_cls(target=target, **thread_kwargs)
    return join(thread.start)




# -*- coding: utf-8 -*-

"""Provide a convienience method to serve a static file from an
  asset specification::
  
      # E.g. in a view callable
      return request.serve_spec('mypkg:foo/bar.js')
  
  Note that the implementation gets the file using its static url, so
  this is only really useful as a way to *obfuscate* the actual url
  that a file is downloaded from, i.e.: if you want to proxy a
  download so as not to expose the actual url to the file.
  
  Note also that this obfuscation is only itself useful if the downloaded
  file is served over HTTPS and has an unguessable file path.
"""

import logging
logger = logging.getLogger(__name__)

import mimetypes
import requests as requests_lib
from cStringIO import StringIO

from pyramid.httpexceptions import HTTPNotFound
from pyramid.response import FileIter
from pyramid.response import Response

def get_serve_spec(request, requests=None, buf_cls=None, iter_cls=None,
        response_cls=None, not_found=None):
    """Return a function that serves an asset specification as a static file."""
    
    # Compose.
    if requests is None:
        requests = requests_lib
    if buf_cls is None:
        buf_cls = StringIO
    if iter_cls is None:
        iter_cls = FileIter
    if response_cls is None:
        response_cls = Response
    if not_found is None:
        not_found = HTTPNotFound
    
    # Prepare.
    not_found_msg = u'The static file could not be found.'
    err_message = u'There was a problem downloading the file. Try again later.'
    
    # Return the serving function.
    def serve(spec):
        """Resolve the asset ``spec`` to a file path and return a static
          file response that serves it. If the file isn't found, return
          a 404.
        """
        
        # Resolve the spec to a url.
        url = request.static_url(spec)
        if url.startswith('//'):
            url = 'https:' + url
        
        # Download the url.
        r = requests.get(url)
        if r.status_code != requests.codes.ok:
            msg = not_found_msg if r.status_code == 404 else err_message
            return not_found(explanation=msg)
        
        # Return the file response.
        filename = spec.split('/')[-1]
        disposition = 'attachment; filename="{0}"'.format(filename)
        mime_type = mimetypes.guess_type(filename)[0] or 'application/octet-stream'
        response = Response(content_type=mime_type)
        response.headers['Content-Disposition'] = disposition
        response.body = r.content
        return response
    
    
    return serve




#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Functional tests for ``pyramid_weblayer``."""

import unittest

try: # pragma: no cover
    from mock import Mock
except: # pragma: no cover
    pass

class TestI18N(unittest.TestCase):
    def setUp(self):
        from pyramid.i18n import Localizer
        self._localizer = Localizer
        self._original_translate = Localizer.translate
        self.mock_translate = Mock()
        self.mock_translate.return_value = 'Bonjour'
        Localizer.translate = self.mock_translate
    
    def tearDown(self):
        self._localizer.translate = self._original_translate
    
    def makeOne(self):
        from os.path import dirname
        from webtest import TestApp
        from pyramid.config import Configurator
        from pyramid.session import UnencryptedCookieSessionFactoryConfig
        session_factory = UnencryptedCookieSessionFactoryConfig('a')
        def test_i18n(request):
            return {}
        config = Configurator(
            settings={'mako.directories': dirname(__file__)},
            session_factory=session_factory
        )
        config.add_route('r1', '/r1')
        config.add_view(test_i18n, route_name='r1', renderer='test_i18n.mako')
        config.include('pyramid_weblayer')
        return TestApp(config.make_wsgi_app())
    
    def test_i18n(self):
        """Making a GET request should work the translation machinery."""
        
        app = self.makeOne()
        res = app.get('/r1', status=200)
        self.mock_translate.assert_called_with('Hello _i18n')
        self.assertTrue('Bonjour'.encode() in res.body)
    


class TestCSRF(unittest.TestCase):
    def makeOne(self):
        from webtest import TestApp
        from pyramid.config import Configurator
        from pyramid.response import Response
        from pyramid.session import UnencryptedCookieSessionFactoryConfig
        session_factory = UnencryptedCookieSessionFactoryConfig('a')
        def test_get(request):
            return Response(request.session.get_csrf_token())
        def test_post(request):
            return Response('posted')
        config = Configurator(settings={}, session_factory=session_factory)
        config.add_route('r1', '/r1')
        config.add_view(test_get, route_name='r1', request_method='GET')
        config.add_view(test_post, route_name='r1', request_method='POST')
        config.include('pyramid_weblayer')
        return TestApp(config.make_wsgi_app())
    
    def test_csrf_token(self):
        """Making a POST with the right CSRF token is OK."""
        
        app = self.makeOne()
        res = app.get('/r1')
        res = app.post('/r1', {'_csrf': res.body})
        self.failUnless('posted'.encode() in res.body)
    
    def test_csrf_invalid_token(self):
        """Making a POST without the right CSRF token fails."""
        
        app = self.makeOne()
        res = app.post('/r2', status=401)
        self.failUnless('Unauthorized'.encode() in res.body)
        res = app.post('/r2', {'_csrf': 'blah'}, status=401)
        self.failUnless('Unauthorized'.encode() in res.body)
    



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Provides py:func`~pyramid_weblayer.utils.generate_hash`` utility function
  to generate seeded or random hashes.
"""

__all__ = [
    'generate_hash',
    'generate_random_digest',
    'get_stamp',
    'datetime_to_float',
]

import logging
logger = logging.getLogger(__name__)

import hashlib
import os
import random
import time

from binascii import hexlify
from datetime import datetime

def generate_hash(s=None, algorithm='sha512', block_size=512):
    """ Generates a :py:func:`~hashlib.hash.hexdigest` string, either randomly
      or from a string or file like object (like an open file or a buffer).
      
      By default, the hash is randomly generated and uses the ``sha512``
      algorithm::
      
          >>> s1 = generate_hash()
          >>> isinstance(s1, str)
          True
          >>> len(s1) == 128
          True
          >>> s2 = generate_hash()
          >>> s1 == s2
          False
          >>> s3 = generate_hash(algorithm='sha512')
          >>> len(s1) == len(s3)
          True
      
      The hash can be generated from a seed::
      
          >>> generate_hash(s='a')
          '1f40fc92da241694750979ee6cf582f2d5d7d28e18335de05abc54d0560e0f5302860c652bf08d560252aa5e74210546f369fbbbce8c12cfc7957b2652fe9a75'
      
      Using ``None`` as the seed (which is the default) will, as we've seen, 
      generate a random value::
      
          >>> s6 = generate_hash(s=None)
          >>> s7 = generate_hash(s=None)
          >>> s6 == s7
          False
      
      Using a file like object (anything with a ``read()`` method) will use
      the contents of the file like object::
      
          >>> from io import BytesIO
          >>> sock = BytesIO()
          >>> s = b'abc'
          >>> l = sock.write(s)
          >>> l = sock.seek(0)
          >>> s8 = generate_hash(s=sock)
          >>> s9 = generate_hash(s=s)
          >>> s8 == s9
          True
      
      Reading the contents into memory in blocks of ``block_size``, which
      defaults to ``512``::
      
          >>> from mock import Mock
          >>> sock = Mock()
          >>> sock.read.return_value = None
          >>> s10 = generate_hash(s=sock)
          >>> sock.read.assert_called_with(512)
          >>> s10 = generate_hash(s=sock, block_size=1024)
          >>> sock.read.assert_called_with(1024)
      
      Using other types as a seed will raise a ``TypeError``::
      
          >>> generate_hash(s=[]) #doctest: +ELLIPSIS
          Traceback (most recent call last):
          ...
          TypeError: ...
      
      The algorithm name can also be passed in::
      
          >>> s4 = generate_hash(algorithm='md5')
          >>> s5 = generate_hash(algorithm='sha224')
          >>> len(s4) == 32 and len(s5) == 56
          True
      
      As long as it's available in :py:mod:`hashlib`::
      
          >>> generate_hash(algorithm='foo')
          Traceback (most recent call last):
          ...
          AttributeError: 'module' object has no attribute 'foo'
      
    """
    
    # get the hasher
    hasher = getattr(hashlib, algorithm)()
    # read in the data
    if hasattr(s, 'read') and callable(s.read):
        while True:
            data = s.read(block_size)
            if not data:
                break
            hasher.update(hasattr(data, 'encode') and data.encode() or data)
    else:
        if s is None:
            s = '%s%s' % (random.random(), time.time())
        if hasattr(s, 'encode') and callable(s.encode):
            hasher.update(s.encode())
        else:
            hasher.update(s)
    # return a hexdigest of the hash
    return hasher.hexdigest()
    

def generate_random_digest(num_bytes=28):
    """Generates a random hash and returns the hex digest as a unicode string.
      
      Defaults to sha224::
      
          >>> import hashlib
          >>> h = hashlib.sha224()
          >>> digest = generate_random_digest()
          >>> len(h.hexdigest()) == len(digest)
          True
      
      Pass in ``num_bytes`` to specify a different length hash::
      
          >>> h = hashlib.sha512()
          >>> digest = generate_random_digest(num_bytes=64)
          >>> len(h.hexdigest()) == len(digest)
          True
      
      Returns unicode::
      
          >>> type(digest) == type(u'')
          True
      
    """
    
    r = os.urandom(num_bytes)
    return unicode(hexlify(r))

def get_stamp(datetime_instance=None, get_now=None):
    """Return a consistent string format for a datetime.
      
      If a ``datetime_instance`` arg is provided, ``str``s it::
      
          >>> get_stamp(True)
          'True'
      
      Otherwise defaults to now::
      
          >>> now = datetime.utcnow()
          >>> mock_get_now = lambda: now
          >>> return_value = get_stamp(get_now=mock_get_now)
          >>> return_value == str(now)
          True
      
    """
    
    if get_now is None:
        get_now = datetime.utcnow
    
    if datetime_instance is None:
        datetime_instance = get_now()
    
    return str(datetime_instance)

def datetime_to_float(datetime_instance):
    """Convert a datetime to as floating point seconds since the epoch, to
      millisecond accuracy.
      
          >>> datetime_to_float(datetime(2000, 1, 1, 1, 1, 1, 1))
          946688461.00000095
      
    """
    
    ts = time.mktime(datetime_instance.timetuple())
    ts += datetime_instance.microsecond / 1000000.0
    return ts




# -*- coding: utf-8 -*-

"""Extend framework using event hooks."""

import logging
logger = logging.getLogger(__name__)

def set_seen_cookie(event):
    """Add a ``has_been_seen_before`` cookie (that lasts for six weeks) to all
      HTML responses.
      
          >>> from mock import Mock
          >>> mock_event = Mock()
      
      Ignores non HTML responses::
      
          >>> mock_event.response.content_type = 'foo'
          >>> set_seen_cookie(mock_event)
          >>> mock_event.response.set_cookie.called
          False
      
      Sets a ``has_been_seen_before`` cookie::
      
          >>> mock_event.response.content_type = 'text/html'
          >>> set_seen_cookie(mock_event)
          >>> mock_event.response.set_cookie.assert_called_with(
          ...         'has_been_seen_before', 'true', max_age=3628800)
      
    """
    
    # Only set the cookie on HTML responses.
    if 'html' in event.response.content_type:
        cookie_name = 'has_been_seen_before'
        expires_after = 3628800
        event.response.set_cookie(cookie_name, 'true', max_age=expires_after)
    

def get_has_been_seen(request):
    """Return ``True`` if the request has the ``has_been_seen_before`` cookie.
      
          >>> from mock import Mock
          >>> mock_request = Mock()
      
      False if the request doesn't have the cookie::
      
          >>> mock_request.cookies = {}
          >>> get_has_been_seen(mock_request)
          False
      
      True if the request does::
      
          >>> mock_request.cookies = {'has_been_seen_before': 'true'}
          >>> get_has_been_seen(mock_request)
          True
      
    """
    
    return bool(request.cookies.get('has_been_seen_before', False))




# -*- coding: utf-8 -*-

"""Utility functions to flatten strings and a template before render hook."""

__all__ = [
    'add_flat_string',
    'as_flat_string',
    'flatten_breadcrumb'
]

import logging
logger = logging.getLogger(__name__)

import html2text

def as_flat_string(markup, to_string=None):
    """Convert html to text and replace all whitespace with single spaces."""
    
    # Compose.
    if to_string is None:
        handler = html2text.HTML2Text()
        handler.ignore_links = True
        handler.ignore_images = True
        to_string = handler.handle
    
    if not markup:
        return markup
    
    s = to_string(markup)
    return u' '.join(s.split()).strip()

def flatten_breadcrumb(markup, to_string=None):
    """Convert html to text and replace bullets and ``/``s."""
    
    # Compose.
    if to_string is None:
        to_string = as_flat_string
    
    if not markup:
        return markup
    
    s = as_flat_string(markup)
    s = s.replace('* ', '').replace('/', '-')
    return s

def add_flatten_functions(event, flat_string=None, flatten_crumb=None):
    """Add ``flat_string`` & ``flatten_breadcrumb`` to the tempate namespace."""
    
    if flat_string is None:
        flat_string = as_flat_string
    if flatten_crumb is None:
        flatten_crumb = flatten_breadcrumb
    
    event['flat_string'] = flat_string
    event['flatten_breadcrumb'] = flatten_breadcrumb




# -*- coding: utf-8 -*-

"""Enforce HTTP Strict Transport Security (see e.g.: `#`_).
  
  _`#`: http://www.imperialviolet.org/2012/07/19/hope9talk.html
"""

# This implementation has been extracted to the `pyramid_hsts` dependency.
# The import api is provided here for backwards compatibility.
from pyramid_hsts import *



# -*- coding: utf-8 -*-

"""Views for ``/favicon.ico`` and ``/robots.txt``."""

import logging
logger = logging.getLogger(__name__)

from pkg_resources import resource_filename

from pyramid.response import FileResponse
from pyramid.security import NO_PERMISSION_REQUIRED as PUBLIC
from pyramid.static import resolve_asset_spec
from pyramid.view import view_config

FAVICON_SPEC = 'pyramid_weblayer:favicon.ico'
ROBOTS_SPEC = 'pyramid_weblayer:robots.txt'

ONE_DAY = 60 * 60 * 24 * 7

def get_absolute_path(spec, resolve=None, filename=None):
    """Turns an asset ``spec`` into an absolute path.
      
      Setup::
      
          >>> from mock import Mock
          >>> mock_filename = Mock()
          >>> mock_filename.return_value = '/var/mypkg/assets/favicon.ico'
      
      Resolves ``spec`` into parts and passes to ``resource_filename``::
      
          >>> get_absolute_path('mypkg:assets/favicon.ico', filename=mock_filename)
          '/var/mypkg/assets/favicon.ico'
          >>> mock_filename.assert_called_with('mypkg', 'assets/favicon.ico')
      
    """
    
    if resolve is None: # pragma: no cover
        resolve = resolve_asset_spec
    if filename is None: # pragma: no cover
        filename = resource_filename
    
    return filename(*resolve(spec))

def serve_file(request, spec, cache_max_age=None, get_path=None, response_cls=None):
    """Serve the ``spec``d file.
      
      Setup::
      
          >>> from mock import Mock
          >>> mock_get_path = Mock()
          >>> mock_get_path.return_value = '/var/foo.txt'
          >>> mock_response_cls = Mock()
          >>> mock_response_cls.return_value = '<response: /var/foo.txt>'
          >>> kwargs = dict(get_path=mock_get_path, response_cls=mock_response_cls)
      
      Gets the absolute path for the ``spec`` and returns the static file
      response::
      
          >>> serve_file('req', 'pkg:foo.txt', **kwargs)
          '<response: /var/foo.txt>'
          >>> mock_get_path.assert_called_with('pkg:foo.txt')
          >>> mock_response_cls.assert_called_with('/var/foo.txt', request='req',
          ...        cache_max_age=ONE_DAY)
      
    """
    
    if cache_max_age is None:
        cache_max_age = ONE_DAY
    if get_path is None: # pragma: no cover
        get_path = get_absolute_path
    if response_cls is None: # pragma: no cover
        response_cls = FileResponse
    
    path = get_path(spec)
    return response_cls(path, request=request, cache_max_age=cache_max_age)


@view_config(route_name='favicon_ico', permission=PUBLIC)
def favicon_view(request, spec=None, serve=None):
    """Serve the ``favicon.ico`` file.
      
          >>> from mock import Mock
          >>> mock_serve = Mock()
          >>> mock_serve.return_value = 'static file response'
          >>> favicon_view('req', serve=mock_serve)
          'static file response'
          >>> mock_serve.assert_called_with('req', FAVICON_SPEC)
      
    """
    
    if spec is None:
        spec = FAVICON_SPEC
    if serve is None: # pragma: no cover
        serve = serve_file
    
    return serve(request, spec)


@view_config(route_name='robots_txt', permission=PUBLIC)
def robots_view(request, spec=None, serve=None):
    """Serve the ``robots.txt`` file.
      
          >>> from mock import Mock
          >>> mock_serve = Mock()
          >>> mock_serve.return_value = 'static file response'
          >>> robots_view('req', serve=mock_serve)
          'static file response'
          >>> mock_serve.assert_called_with('req', ROBOTS_SPEC)
      
    """
    
    if spec is None:
        spec = ROBOTS_SPEC
    if serve is None: # pragma: no cover
        serve = serve_file
    
    return serve(request, spec)




# -*- coding: utf-8 -*-

"""Provides ``humanize_time`` function."""

__all__ = [
    'humanize_time',
]

import logging
logger = logging.getLogger(__name__)

from datetime import datetime

INTERVALS = [1, 60, 3600, 86400, 604800, 2419200, 29030400]
NAMES = [
    ('second', 'seconds'),
    ('minute', 'minutes'),
    ('hour',   'hours'),
    ('day',    'days'),
    ('week',   'weeks'),
    ('month',  'months'),
    ('year',   'years'),
]

def humanize_time(amount, units):
    """Divide `amount` in time periods. Useful for making time intervals more
      human readable.
      
          >>> humanize_time(173, "hours")
          [(1, 'week'), (5, 'hours')]
          >>> humanize_time(17313, "seconds")
          [(4, 'hours'), (48, 'minutes'), (33, 'seconds')]
          >>> humanize_time(90, "weeks")
          [(1, 'year'), (10, 'months'), (2, 'weeks')]
          >>> humanize_time(42, "months")
          [(3, 'years'), (6, 'months')]
          >>> humanize_time(500, "days")
          [(1, 'year'), (5, 'months'), (3, 'weeks'), (3, 'days')]
      
      See http://stackoverflow.com/a/6574789 for original by Liudmil Mitev.
    """
    
    result = []
    unit = map(lambda a: a[1], NAMES).index(units)
    # Convert to seconds
    amount = amount * INTERVALS[unit]
    for i in range(len(NAMES)-1, -1, -1):
        a = amount // INTERVALS[i]
        if a > 0: 
            result.append( (a, NAMES[i][1 % a]) )
            amount -= a * INTERVALS[i]
    return result




#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Provides a ``validate_against_csrf`` ``pyramid.events.ContextFound`` subscriber
  that uses a ``CSRFValidator`` utility to validate incoming requests against
  cross site request forgeries.
"""

__all__ = [
    'CSRFError',
    'CSRFValidator',
    'METHODS_WITH_SIDE_EFFECTS',
    'validate_against_csrf',
    'validate_session_authenticated',
]

import logging
logger = logging.getLogger(__name__)

from pyramid.httpexceptions import HTTPUnauthorized
from pyramid.interfaces import IAuthenticationPolicy
from pyramid.security import unauthenticated_userid
from pyramid_layout.panel import panel_config

METHODS_WITH_SIDE_EFFECTS = (
    'delete',
    'post', 
    'put',
)

class CSRFError(ValueError):
    """Raised when csrf validation fails."""


class CSRFValidator(object):
    """Validate a request against cross site request forgeries."""
    
    def __init__(self, session_token, target_methods=METHODS_WITH_SIDE_EFFECTS):
        self._session_token = session_token
        self._target_methods = target_methods
    
    def validate(self, request):
        if not request.method.lower() in self._target_methods:
            return
        if request.is_xhr:
            default_value = request.headers.get('X-CSRFToken', None)
        else:
            default_value = None
        token_value = request.params.get('_csrf', default_value)
        if token_value is None or token_value != self._session_token:
            raise CSRFError
    


def validate_against_csrf(event, validator_cls=None):
    """Event subscriber that uses the session to validate incoming requests."""
    
    # Compose.
    if validator_cls is None:
        validator_cls = CSRFValidator
    
    # Unpack.
    request = event.request
    settings = request.registry.settings
    
    # Only validate if enabled.
    if not settings.get('csrf.validate', True):
        return
    
    # Ignore specified routes.
    matched_route = request.matched_route
    ignore_routes = settings.get('csrf.ignore_routes', None)
    if matched_route and ignore_routes:
        if matched_route.name in ignore_routes.split():
            return
    
    # Ignore specified paths.
    ignore_paths = settings.get('csrf.ignore_paths', None)
    if ignore_paths:
        for path in ignore_paths.split():
            if request.path.startswith(path):
                return
    
    session_token = request.session.get_csrf_token()
    csrf_validator = validator_cls(session_token)
    try:
        csrf_validator.validate(request)
    except CSRFError:
        raise HTTPUnauthorized

def validate_session_authenticated(event, unauth_userid=None, validate=None):
    """Event subscriber that validates *session authenticated* requests
      against Cross Site Request Forgery.
    """
    
    # Compose.
    if unauth_userid is None:
        unauth_userid = unauthenticated_userid
    if validate is None:
        validate = validate_against_csrf
    
    # Unpack.
    request = event.request
    registry = request.registry
    
    # Start by making CSRF validation the default. This protects against
    # exposure to having coded against the internal implementation detail
    # noted below -- if something changes and this logic breaks, we'll
    # hopefully deny requests we should allow, rather than allowing requests
    # we should deny.
    should_validate = True
    
    # Determine whether to validate the request -- first checking that the
    # request *is* authenticated, then that we're not running tests,
    # then that it was authenticated using a policy that is (or is like)
    # the pyramid ``SessionAuthenticationPolicy``.
    if not request.is_authenticated:
        should_validate = False
    elif request.environ.get('paste.testing'):
        should_validate = False
    else: # XXX this next logic is coded against an internal implementation
        # detail of the ``SessionAuthenticationPolicy``.
        policy = registry.queryUtility(IAuthenticationPolicy)
        key = getattr(policy, 'userid_key', None)
        session_value = key and request.session.get(key) or None
        principal_id = unauth_userid(request)
        # We know that the principal_id exists as the request is authenticated,
        # so checking that the session_value equals it also checks that the
        # session value is not None.
        should_validate = session_value == principal_id
    if should_validate:
        validate(event)


@panel_config('csrf-ajax-setup',
        renderer='pyramid_weblayer:templates/csrf_ajax_setup.mako')
def csrf_ajax_setup_panel(context, request):
    """Pass the current CSRF token and target methods to the panel template."""
    
    return {
        'token': request.session.get_csrf_token(), 
        'methods': [item.upper() for item in METHODS_WITH_SIDE_EFFECTS]
    }




#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Provides an ``add_underscore_translation`` ``pyramid.events.BeforeRender``
  subscriber that adds a ``_`` translation function to the template global
  namespace.
"""

from pyramid.i18n import get_localizer, TranslationStringFactory

class TranslationAdapter(object):
    """Adapt a ``request`` to provide ``self.translate(message_string)``, e.g.::
    
      setUp::
      
          >>> from mock import Mock
          >>> mock_request = Mock()
          >>> mock_request.localizer.translate.return_value = 'manger de tout'
      
      Example usage::
          
          >>> translator = TranslationAdapter(mock_request)
          >>> translator.translate('eat everything')
          'manger de tout'
      
    """
    
    def __init__(self, request, domain=None):
        self.localizer = get_localizer(request)
        self.factory = TranslationStringFactory(domain)
    
    def translate(self, message_string):
        return self.localizer.translate(self.factory(message_string))
    


def add_underscore_translation(event, Adapter=TranslationAdapter):
    """Add i18n support to the template global namespace.
      
      setUp::
      
          >>> from mock import Mock
          >>> MockAdapter = Mock()
          >>> mock_translator = Mock()
          >>> mock_translator.translate.return_value = 'manger de tout'
          >>> MockAdapter.return_value = mock_translator
          >>> event = {'request': None}
      
      Example usage::
      
          >>> add_underscore_translation(event, Adapter=MockAdapter)
          >>> translate = event['_']
          >>> translate('eat everything')
          'manger de tout'
      
    """
    
    translator = Adapter(event['request'])
    event['_'] = translator.translate
    event['localizer'] = translator.localizer




#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Provides a request hook that can be used to generate a google analytics
 trackable route url.
"""

def get_campaign_url(request):
    """Returns a function that will generate a trackable campaign url."""
    
    def campaign_url(spec, source=u'', medium=u'', term=u'', content=u'', campaign=u'', **kwargs):
        """Encode the kwargs as query params"""
        
        # Either create a new query or extend the existing one.
        if '_query' in kwargs:
            query = kwargs.pop('_query')
        else:
            query = {}
        
        # Append the google analytics tracking params, if provided.
        if source:
            query['utm_source'] = source
        if medium:
            query['utm_medium'] = medium
        if term:
            query['utm_term'] = term
        if content:
            query['utm_content'] = content
        if campaign:
            query['utm_campaign'] = campaign
        return request.route_url(spec, _query=query, **kwargs)
    
    return campaign_url




# -*- coding: utf-8 -*-

"""Utility functions to snip text at a certain length."""

import logging
logger = logging.getLogger(__name__)

import html2text

def snip_text(text, n=140):
    """Snip text at word boundary. Defaults to 140 characters.
      
          >>> text = u'lorum ipsum dolores. ' * 10
          >>> snip_text(text)
          u'lorum ipsum dolores. lorum ipsum dolores. lorum ipsum dolores. lorum ipsum dolores. lorum ipsum dolores. lorum ipsum dolores. lorum ipsum dolores. lorum ipsum \u2026'
          >>> snip_text(u'lorum ipsum dolores')
          u'lorum ipsum dolores'
          >>> snip_text(u'lorum ipsum dolores', n=8)
          u'lorum \u2026'
      
      Anything that evaluates to False comes out as an empty string::
      
          >>> snip_text(None)
          u''
      
    """
    
    if not text:
        return u''
    
    if len(text) < n:
        return text
    
    i = 0
    words = []
    for item in text.split():
        l = len(item)
        if i + l < n:
            words.append(item)
            i += l
        else:
            break
    return u'{0} …'.format(u' '.join(words))

def snip_html(html, n=140, handler=None, snip=None):
    """Snip html => raw text at word boundary.
      
          >>> html = 'Hello <em>is it me</em> you are looking for? ' * 10
          >>> snip_html(html)
          u'Hello _is it me_ you are looking for? Hello _is it me_ you are looking for? Hello _is it me_ you are looking for? Hello _is it me_ you are looking for? Hello _is it me_ you are \u2026'
          >>> snip_html(html, n=8)
          u'Hello \u2026'
      
    """
    
    # Test jig.
    if handler is None:
        handler = html2text.HTML2Text()
        handler.ignore_links = True
        handler.ignore_images = True
    if snip is None:
       snip = snip_text
    
    # Turn the html into plain text.
    text = handler.handle(html)
    
    # Run it through the snip text function.
    return snip(text, n=n)


def add_snip_functions(event):
    """Add ``snip_text`` and ``snip_html`` functions to the template namespace.
      
          >>> event = {}
          >>> add_snip_functions(event)
          >>> event['snip_text'] == snip_text
          True
          >>> event['snip_html'] == snip_html
          True
      
    """
    
    event['snip_text'] = snip_text
    event['snip_html'] = snip_html




# -*- coding: utf-8 -*-

"""Provides the ``main()`` WSGI application entry point."""

import logging
logger = logging.getLogger(__name__)

import os

from pyramid.config import Configurator

from pyramid_basemodel import Session
from pyramid_weblayer.patch import patch_all

def augment_settings(settings, env):
    """Use the ``env`` to augment the ``settings``."""
    
    # Extend the settings with the INI_* environment variables.
    for k, v in env.items():
        if k.startswith('INI_'):
            k = k[4:].replace('__', '.')
            settings[k] = v
    
    # Use the env `DATABASE_URL` as our `sqlalchemy.url`.
    if env.has_key('DATABASE_URL') or env.has_key('SHARED_DATABASE_URL'):
        settings['sqlalchemy.url'] = env.get('DATABASE_URL', 
                env.get('SHARED_DATABASE_URL'))
    
    # Use the memcache settings for the beaker backend.
    has_memcache = False
    memcache_stub = None
    for item in ('MEMCACHIER', 'MEMCACHE'):
        if env.has_key('{0}_SERVERS'.format(item)):
            has_memcache = True
            memcache_stub = item
            break
    if has_memcache:
        mc_username_key = '{0}_USERNAME'.format(memcache_stub)
        mc_password_key = '{0}_PASSWORD'.format(memcache_stub)
        mc_servers_key = '{0}_SERVERS'.format(memcache_stub)
        # Check the ``type`` config for the session and the template cache.
        # We want to set the url etc. for ext types.
        namespaces = []
        if settings['session.type'].startswith('ext:'):
            namespaces.append('session.')
        if settings['mako.cache_args.type'].startswith('ext:'):
            namespaces.append('mako.cache_args.')
        # Build the data to set.
        data = {'url': env[mc_servers_key]}
        if env.has_key(mc_username_key) and env.has_key(mc_password_key):
            data['username'] = env[mc_username_key]
            data['password'] = env[mc_password_key]
            data['protocol'] = 'binary'
        # Update the settings.
        for ns in namespaces:
            for k, v in data.items():
                settings['{0}{1}'.format(ns, k)] = v
    
    # Forcibly coerce all setting values that can be coerced to integers.
    for k, v in settings.items():
        try:
            settings[k] = int(v)
        except ValueError:
            pass
    
    # Return, augmented.
    return settings

def make_wsgi_app(root_factory, includeme, patch=None, bind=None, augment=None,
        env=None, configurator_cls=None, session=None, **settings):
    """Create and return a WSGI application."""
    
    # Compose.
    if augment is None: 
        augment = augment_settings
    if env is None:
        env = os.environ
    if configurator_cls is None:
        configurator_cls = Configurator
    if session is None:
        session = Session
    
    # If we should augment the settings with the environment variables, do so.
    if augment is not None:
        settings = augment(settings, env)
    
    # Now we have the settings, run the patches.
    if patch is not None:
        patch(settings)
    
    # Bind the db model.
    if bind is not None:
        bind()
    
    # Initialise a ``Configurator`` and apply the package configuration.
    config = configurator_cls(settings=settings, root_factory=root_factory)
    includeme(config)
    
    # Close the db connection for this thread.
    session.remove()
    
    # Return a WSGI app.
    return config.make_wsgi_app()




# -*- coding: utf-8 -*-

"""Provide a ``request.track_event(category, action, label=None, value=None)``
  method which fires the equivalent of a client side ``ga._trackEvent(...)``
  request to google analytics.
  
  The request is joined to the transaction and made in a background thread.
"""

__all__ = [
    'get_track_event'
]

import logging
logger = logging.getLogger(__name__)

from pyga.entities import Event
from pyga.entities import Page
from pyga.entities import Session
from pyga.entities import Visitor
from pyga.requests import Tracker

from .tx import call_in_background

class PyGAFactory(object):
    """Shared boilerplate to get configured ``tracker, session, visitor``
      instances from the current request::
      
      Setup::
      
          >>> from mock import Mock
          >>> mock_request = Mock()
          >>> mock_request.registry.settings = {'gae.tracking_id': '...'}
          >>> mock_request.session = {}
          >>> mock_session_cls = Mock()
          >>> mock_tracker_cls = Mock()
          >>> mock_visitor_cls = Mock()
          >>> mock_kwargs = dict(session_cls=mock_session_cls,
          ...         tracker_cls=mock_tracker_cls, visitor_cls=mock_visitor_cls)
      
      Test::
      
          >>> factory = PyGAFactory(**mock_kwargs)
          >>> tracker, session, visitor = factory(mock_request)
          >>> assert tracker is mock_tracker_cls.return_value
          >>> assert session is mock_session_cls.return_value
          >>> assert visitor is mock_visitor_cls.return_value
      
    """
    
    def __call__(self, request):
        """Instantiate and return pyga ``tracker, session, visitor`` instances."""
        
        # Prepare the tracking id, session id and utm cookies.
        settings = request.registry.settings
        gae_tracking_id = settings['gae.tracking_id']
        if not 'gae_session_id' in request.session:
            request.session['gae_session_id'] = self.session_cls.generate_session_id()
        gae_session_id = request.session['gae_session_id']
        utma_cookie = request.cookies.get('__utma', None)
        utmb_cookie = request.cookies.get('__utmb', None)
        
        # Instantiate the tracker, session and visitor.
        tracker = self.tracker_cls(gae_tracking_id, request.host)
        session = self.session_cls()
        session.session_id = gae_session_id
        if utmb_cookie:
            session.extract_from_utmb(utmb_cookie)
        visitor = self.visitor_cls()
        visitor.extract_from_server_meta(request.environ)
        if utma_cookie:
            visitor.extract_from_utma(utma_cookie)
        
        # Return the tuple.
        return tracker, session, visitor
    
    def __init__(self, session_cls=None, tracker_cls=None, visitor_cls=None):
        """Initialise with the dependencies provided."""
        
        # Compose.
        if session_cls is None: #pragma: no cover
            session_cls = Session
        if tracker_cls is None: #pragma: no cover
            tracker_cls = Tracker
        if visitor_cls is None: #pragma: no cover
            visitor_cls = Visitor
        
        # Assign.
        self.session_cls = session_cls
        self.tracker_cls = tracker_cls
        self.visitor_cls = visitor_cls
    


def get_track_event(request, call_in_bg=None, event_cls=None, factory_cls=None):
    """Return a request method that uses the pyga library to track a custom
      event with google analytics.
      
      Setup::
      
          >>> from mock import Mock
          >>> mock_call_in_bg = Mock()
          >>> mock_event_cls = Mock()
          >>> mock_event_cls.return_value = '<event>'
          >>> mock_factory_cls = Mock()
          >>> mock_factory = Mock()
          >>> mock_tracker = Mock()
          >>> mock_factory.return_value = (mock_tracker, '<session>', '<visitor>')
          >>> mock_factory_cls.return_value = mock_factory
          >>> mock_request = Mock()
          >>> track_event = get_track_event(mock_request,
          ...         call_in_bg=mock_call_in_bg, event_cls=mock_event_cls,
          ...         factory_cls=mock_factory_cls)
      
      Tracks the event::
      
          >>> return_value = track_event('cat', 'act')
          >>> args = ('<event>', '<session>', '<visitor>')
          >>> mock_call_in_bg.assert_called_with(mock_tracker.track_event, args=args)
      
      Unless in development::
      
          >>> mock_request.registry.settings = {'mode': 'development'}
          >>> mock_call_in_bg = Mock()
          >>> track_event = get_track_event(mock_request,
          ...         call_in_bg=mock_call_in_bg, event_cls=mock_event_cls,
          ...         factory_cls=mock_factory_cls)
          >>> return_value = track_event('cat', 'act')
          >>> assert not mock_call_in_bg.called
      
    """
    
    # Compose.
    if call_in_bg is None: #pragma: no cover
        call_in_bg = call_in_background
    if event_cls is None: #pragma: no cover
        event_cls = Event
    if factory_cls is None: #pragma: no cover
        factory_cls = PyGAFactory
    
    def track_event(category, action, label=None, value=None, noninteraction=False):
        """Join a call to ping google analytics with the given event in a background thread
          to the current transaction.
        """
        
        # Exit if in development.
        settings = request.registry.settings
        if settings.get('mode', None) in ['development', 'testing']:
            return
        
        # Instantiate configured pyga ``tracker``, ``session`` and ``visitor``s.
        factory = factory_cls()
        tracker, session, visitor = factory(request)
        
        # Create the event to track.
        event = event_cls(category=category, action=action, label=label,
                value=value, noninteraction=noninteraction)
        
        # Join a background call to track the event to the current transaction.
        return call_in_bg(tracker.track_event, args=(event, session, visitor))
    
    return track_event

def get_track_page(request, call_in_bg=None, page_cls=None, factory_cls=None):
    """Return a request method that uses the pyga library to track a page view
      (or a virtual page view) with google analytics.
      
      Setup::
      
          >>> from mock import Mock
          >>> mock_call_in_bg = Mock()
          >>> mock_page_cls = Mock()
          >>> mock_page_cls.return_value = '<page>'
          >>> mock_factory_cls = Mock()
          >>> mock_factory = Mock()
          >>> mock_tracker = Mock()
          >>> mock_factory.return_value = (mock_tracker, '<session>', '<visitor>')
          >>> mock_factory_cls.return_value = mock_factory
          >>> mock_request = Mock()
          >>> track_page = get_track_page(mock_request,
          ...         call_in_bg=mock_call_in_bg, page_cls=mock_page_cls,
          ...         factory_cls=mock_factory_cls)
      
      Tracks the page view::
      
          >>> return_value = track_page('/foo?baz=bar')
          >>> mock_page_cls.assert_called_with('/foo?baz=bar')
          >>> args = ('<page>', '<session>', '<visitor>')
          >>> mock_call_in_bg.assert_called_with(mock_tracker.track_pageview, args=args)
      
      Unless in development::
      
          >>> mock_request.registry.settings = {'mode': 'development'}
          >>> mock_call_in_bg = Mock()
          >>> track_page = get_track_page(mock_request,
          ...         call_in_bg=mock_call_in_bg, page_cls=mock_page_cls,
          ...         factory_cls=mock_factory_cls)
          >>> return_value = track_page('/foo?baz=bar')
          >>> assert not mock_call_in_bg.called
      
    """
    
    # Compose.
    if call_in_bg is None: #pragma: no cover
        call_in_bg = call_in_background
    if page_cls is None: #pragma: no cover
        page_cls = Page
    if factory_cls is None: #pragma: no cover
        factory_cls = PyGAFactory
    
    def track_page(path):
        """Join a call to ping google analytics with the given event in a background thread
          to the current transaction.
        """
        
        # Exit if in development.
        settings = request.registry.settings
        if settings.get('mode', None) in ['development', 'testing']:
            return
        
        # Instantiate configured pyga ``tracker``, ``session`` and ``visitor``s.
        factory = factory_cls()
        tracker, session, visitor = factory(request)
        
        # Create the page to track.
        page = page_cls(path)
        
        # Join a background call to track the event to the current transaction.
        return call_in_bg(tracker.track_pageview, args=(page, session, visitor))
    
    return track_page




#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Provides py:func`~pyramid_weblayer.nav.is_active`` function that determines
  whether a navigation item is active based on the request and path provided.
"""

import logging
logger = logging.getLogger(__name__)

class ActiveNavigationAdapter(object):
    """Adapt a ``request`` to provide ``self.is_active(path)``.
      
      Setup::
      
          >>> from mock import Mock
          >>> mock_request = Mock()
          >>> mock_request.path = '/foo/bar'
      
      Returns the string ``'active'`` if True, otherwise returns an empty string::
          
          >>> nav = ActiveNavigationAdapter(mock_request)
          >>> nav.is_active('/foo')
          'active'
          >>> nav.is_active('/foo/baz')
          ''
      
      The idea being that the return value can be used in a html element's class
      attribute::
      
          <li class="nav-item ${is_active('/foo')}">
            <a href="/foo">Foo</a>
          </li>
      
      Can accept multiple (OR) paths::
      
          >>> nav.is_active('/foo/baz', '/flobble', '/bar')
          ''
          >>> nav.is_active('/foo/baz', '/foo')
          'active'
          >>> nav.is_active('/foo', '/foo/baz')
          'active'
      
      Special cases '/' to require an exact match::
      
          >>> nav.is_active('/')
          ''
          >>> mock_request.path = '/'
          >>> nav = ActiveNavigationAdapter(mock_request)
          >>> nav.is_active('/')
          'active'
      
    """
    
    def __init__(self, request):
        self.request = request
    
    def is_active(self, *args, **kwargs):
        exact = kwargs.get('exact')
        for path in args:
            if exact is None:
                exact = path == '/'
            active = False
            if exact:
                active = self.request.path == path
            else:
                active = self.request.path.startswith(path)
            if active:
                return 'active'
        return ''
    


def add_is_active_function(event, adapter_cls=None):
    """Add the ``is_active`` function to the template global namespace.
      
      Setup::
      
          >>> from mock import Mock
          >>> mock_adapter_cls = Mock()
          >>> mock_adapter = Mock()
          >>> mock_adapter.is_active.return_value = True
          >>> mock_adapter_cls.return_value = mock_adapter
          >>> event = {'request': None}
      
      Adds ``as_active`` to the event namespace::
      
          >>> add_is_active_function(event, adapter_cls=mock_adapter_cls)
          >>> is_active = event['is_active']
          >>> is_active('/')
          True
          >>> mock_adapter.is_active.assert_called_with('/')
      
    """
    
    if adapter_cls is None:
        adapter_cls = ActiveNavigationAdapter
    
    nav = adapter_cls(event['request'])
    event['is_active'] = nav.is_active




# -*- coding: utf-8 -*-

"""Provide a ``request.joined_flash`` method which joins a call to
  ``request.session.flash`` to the current transaction.
"""

__all__ = [
    'get_joined_flash'
]

import logging
logger = logging.getLogger(__name__)

from .tx import join_to_transaction

def get_joined_flash(request, join=None):
    """Return the ``joined_flash`` request method."""
    
    # Compose.
    if join is None:
        join = join_to_transaction
    
    def joined_flash(*args, **kwargs):
        """Join a call to ``request.session.flash`` to the current transaction."""
        
        return join(request.session.flash, *args, **kwargs)
    
    return joined_flash




#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Unit tests for `pyramid_weblayer.csrf`."""

import unittest

try: # pragma: no cover
    from mock import Mock
except: # pragma: no cover
    pass

class TestCSRFValidator(unittest.TestCase):
    """Test the logic of py:class:`~pyramid_weblayer.csrf.CSRFValidator`."""
    
    def makeOne(self, session_token, **kwargs):
        from ..csrf import CSRFValidator
        return CSRFValidator(session_token, **kwargs)
    
    def test_validates_with_correct_request_param(self):
        """Requests validate OK when they have the correct ``_csrf``."""
        
        from ..csrf import CSRFError
        
        mock_request = Mock()
        mock_request.method = 'a'
        mock_request.params = {'_csrf': 'token'}
        
        validator = self.makeOne('token', target_methods=['a'])
        retval = validator.validate(mock_request)
        self.assertTrue(retval is None)
    
    def test_raises_csrf_error_with_no_request_param(self):
        """Requests must have ``_csrf``."""
        
        from ..csrf import CSRFError
        
        mock_request = Mock()
        mock_request.method = 'a'
        mock_request.params = {}
        
        validator = self.makeOne('token', target_methods=['a'])
        self.assertRaises(
            CSRFError,
            validator.validate,
            mock_request
        )
    
    def test_raises_csrf_error_with_wrong_request_param(self):
        """Requests must have the right ``_csrf``."""
        
        from ..csrf import CSRFError
        
        mock_request = Mock()
        mock_request.method = 'a'
        mock_request.params = {'_csrf': 'wrong'}
        
        validator = self.makeOne('token', target_methods=['a'])
        self.assertRaises(
            CSRFError,
            validator.validate,
            mock_request
        )
    
    def test_method_must_have_side_effects(self):
        """Method must have side effects."""
        
        from ..csrf import CSRFError
        
        mock_request = Mock()
        mock_request.method = 'b'
        
        validator = self.makeOne('token', target_methods=['a'])
        retval = validator.validate(mock_request)
        
        self.assertTrue(retval is None)
        self.assertRaises(
            AssertionError,
            mock_request.headers.get.assert_called_with,
            'X-Requested-With'
        )
        
        mock_request = Mock()
        mock_request.method = 'a'
        self.assertRaises(
            CSRFError,
            validator.validate,
            mock_request
        )
    
    def test_force_to_lowercase(self):
        """Method name is forced to lower case."""
        
        from ..csrf import CSRFError
        
        mock_request = Mock()
        mock_request.method = 'A'
        
        validator = self.makeOne('token', target_methods=['a'])
        self.assertRaises(
            CSRFError,
            validator.validate,
            mock_request
        )
    
    def test_xmlhttprequests(self):
        """Also validates ``XMLHttpRequest``s."""
        
        from ..csrf import CSRFError
        
        mock_request = Mock()
        mock_request.method = 'A'
        
        validator = self.makeOne('token', target_methods=['a'])
        self.assertRaises(
            CSRFError,
            validator.validate,
            mock_request
        )
        
        mock_request.headers = {'X-Requested-With': 'XMLHttpRequest'}
        self.assertRaises(
            CSRFError,
            validator.validate,
            mock_request
        )
    
    def test_xmlhttprequests_headers(self):
        """Looks for token in ``X-CSRFToken`` of ``XMLHttpRequest``s."""
        
        from ..csrf import CSRFError
        
        mock_request = Mock()
        mock_request.method = 'A'
        mock_request.headers = {
            'X-Requested-With': 'XMLHttpRequest',
            'X-CSRFToken': 'token'
        }
        validator = self.makeOne('token', target_methods=['a'])
        retval = validator.validate(mock_request)
        self.assertTrue(retval is None)
    
    def test_xmlhttprequests_headers(self):
        """``X-CSRFToken`` will be overriden by ``_csrf``."""
        
        from ..csrf import CSRFError
        
        mock_request = Mock()
        mock_request.method = 'A'
        mock_request.headers = {
            'X-Requested-With': 'XMLHttpRequest',
            'X-CSRFToken': 'wrong'
        }
        validator = self.makeOne('token', target_methods=['a'])
        self.assertRaises(
            CSRFError,
            validator.validate,
            mock_request
        )
        
        mock_request.params = {'_csrf': 'token'}
        validator = self.makeOne('token', target_methods=['a'])
        retval = validator.validate(mock_request)
        self.assertTrue(retval is None)
    


class TestCSRFSubscriber(unittest.TestCase):
    """Test the logic of ``pyramid_weblayer.csrf.validate_against_csrf``."""
    
    def test_validates_the_request(self):
        """Validates the request against CSRF attacks."""
        
        from ..csrf import validate_against_csrf
        
        mock_request = Mock()
        mock_request.registry.settings = {}
        mock_validator = Mock()
        mock_validator_factory = Mock()
        mock_validator_factory.return_value = mock_validator
        mock_event = Mock()
        mock_event.request = mock_request
        
        validate_against_csrf(mock_event, Validator=mock_validator_factory)
        mock_validator.validate.assert_called_with(mock_request)
    
    def test_doesnt_validate_the_request(self):
        """Only validates the request against CSRF attacks if settings['csrf_validate']
          isn't false.
        """
        
        from ..csrf import validate_against_csrf
        
        mock_request = Mock()
        mock_request.registry.settings = {'csrf.validate': False}
        mock_validator = Mock()
        mock_validator_factory = Mock()
        mock_validator_factory.return_value = mock_validator
        mock_event = Mock()
        mock_event.request = mock_request
        
        validate_against_csrf(mock_event, Validator=mock_validator_factory)
        self.assertRaises(
            AssertionError,
            mock_validator.validate.assert_called_with,
            mock_request
        )
    
    def test_raises_httpunauthorized(self):
        """Raises ``HTTPUnauthorized`` if CSRF doesn't validate."""
        
        from pyramid.httpexceptions import HTTPUnauthorized
        from ..csrf import validate_against_csrf, CSRFError
        
        def raise_exc(*args):
            raise CSRFError
        
        mock_request = Mock()
        mock_request.registry.settings = {}
        mock_validator = Mock()
        mock_validator_factory = Mock()
        mock_validator_factory.return_value = mock_validator
        mock_validator.validate = raise_exc
        mock_event = Mock()
        mock_event.request = mock_request
        
        self.assertRaises(
            HTTPUnauthorized,
            validate_against_csrf,
            mock_event, 
            Validator=mock_validator_factory
        )
    

